package common;

import java.awt.Color;
import java.util.Comparator;

public class Compare {
	public static final double epsilon = 0.00000001;
	
	// compare(i1, i2)
	//   i1: int = an integer
	//   i2: int = another integer
	// the usual comparison function on integers
	public static int compare(int i1, int i2) 
	{
		if (i1 == i2) return 0;
		if (i1 < i2) return -1;
		else return 1;
	}

	// compare(f1, f2)
	//   f1: float = a float
	//   f2: float = another float
	// the usual comparison function on floats, with a fixed precision
	public static int compare(float f1, float f2)
	{
		double diff = (double)f1 - (double)f2;
		if (Math.abs(diff) < epsilon) return 0;
		if (diff < 0) return -1;
		return 1;
	}
	public static boolean equals(float f1, float f2)
	{ return compare(f1, f2) == 0; }
	public static boolean less(float f1, float f2)
	{ return compare(f1, f2) == -1; }

	
	// compare(d1, d2)
	//   d1: double = a double
	//   d2: double = another double
	// the usual comparison function on doubles, with a fixed precision
	public static int compare(double d1, double d2)
	{
		double diff = d1 - d2;
		if (Math.abs(diff) < epsilon) return 0;
		if (diff < 0) return -1;
		return 1;
	}
	public static boolean equals(double d1, double d2)
	{ return compare(d1, d2) == 0; }
	public static boolean less(double d1, double d2)
	{ return compare(d1, d2) == -1; }
	
	// compareRGB(c1, c2)
	//   c1: Color = a color
	//   c2: Color = a color
	// orders colors first by red value, then green, then blue
	public static int compareRGB(Color c1, Color c2)
	{
		int rComp = compare(c1.getRed(), c2.getRed());
		int gComp = compare(c1.getGreen(), c2.getGreen());
		int bComp = compare(c1.getBlue(), c2.getBlue());
		if (rComp != 0) return rComp;
		if (gComp != 0) return gComp;
		if (bComp != 0) return bComp;
		return 0;
	}

	// compareHSB(c1, c2)
	//   c1: Color = a color
	//   c2: Color = a color
	// orders colors first by hue, then by saturation, then by brightness	
	public static int compareHSB(Color c1, Color c2)
	{
		float HSB1[] = Color.RGBtoHSB(c1.getRed(), c1.getGreen(), c1.getBlue(), null);
		float hue1 = HSB1[0],  sat1 = HSB1[1],  brt1 = HSB1[2];
		float HSB2[] = Color.RGBtoHSB(c2.getRed(), c2.getGreen(), c2.getBlue(), null);
		float hue2 = HSB2[0],  sat2 = HSB2[1],  brt2 = HSB2[2];
		int hueComp = compare(hue1, hue2);
		int satComp = compare(sat1, sat2);
		int brtComp = compare(brt1, brt2);
		if (hueComp != 0) return hueComp;
		if (satComp != 0) return satComp;
		if (brtComp != 0) return brtComp;
		return 0;
	}

	public static Comparator<Color> comparatorHSB()
	{
		return new Comparator<Color>() {
			@Override
			public int compare(Color c1, Color c2) {
				return compareHSB(c1, c2);
			}
		};
	}
}
