package common;
import java.awt.Color;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;



public class PastelColorMap implements ColorMap {

	private static TreeMap<String, Set<String>> _categories;

	// PastelColorMap(categories)
	//   categories: Map<String, List<String>> = map of categories to skills
	// create color map with given category meta-data
	public PastelColorMap(TreeMap<String, List<String>> categories)
	{
		_categories = new TreeMap<String, Set<String>>();
		for (Map.Entry<String, List<String>> kv : categories.entrySet()) {
			String cat = kv.getKey();
			List<String> contents = kv.getValue();
			for (String skill : contents) {
				if (_categories.containsKey(skill)) {
					_categories.get(skill).add(cat);
				}
				else _categories.put(skill, new TreeSet<String>
				(Arrays.asList(cat)));
			}
		}
	}

	
	@Override
	public Color colorOf(String id)
	{
		return colorMap(id);
	}
	
	// colorMap(skill)
	//   skill: String = a skill name
	// returns color associated with given skill, or black if none defined
	public Color colorMap(String skill) 
	{
		Color orange = new Color(255, 173, 84);
		Color red = new Color(255, 71, 71);
		Color blue = new Color(176, 180, 224);
		Color lightblue = new Color(172, 231, 255);
		Color darkblue = new Color(119, 174, 236);
		Color green = new Color(104, 148, 86);
		Color yellow = new Color(255, 243, 104);
		Color unknown = Color.white;

		if (_categories == null) return unknown;
		
		// get first category that matches skill
		Set<String> cats = _categories.get(skill);
		String cat = "unknown";
		if (cats != null && !cats.isEmpty()) cat = cats.iterator().next();

		// return color for given categories
		if (cat == "Languages") return orange;
		if (cat == "IT Systems") return red;
		if (cat == "Graphics") return green;
		if (cat == "Web Design") return blue;
		//if (cat == "Windows") return yellow;
		if (cat == "Windows") return Color.white;
		if (cat == "Misc") return Color.white;
		
		// return color for hard-coded skills with no category
		if (skill == "BSCS") return lightblue;
		if (skill == "MSCS") return darkblue;
		if (skill == "") return unknown;     // chart placeholder
		
		// return color for unrecognized category
		System.out.println("Colormap error: no color value defined for skill: '" 
				+ skill + "' in category: '" + cat + "'");
		return unknown;
	}
}
