package geometry;

import java.awt.Shape;
import java.awt.geom.Ellipse2D;

import common.Compare;

public class Point implements Comparable<Point> {
	public static float epsilon = 0.0000001f;
	public float x = 0;
	public float y = 0;	

	// plus(p, q)
	//   p: Point = a point
	//   q: Point = another point
	// returns new Point equal to the sum of p and q
	public static Point plus(Point p, Point q)
	{
		return new Point(p.x+q.x, p.y+q.y);
	}
	
	public Point() {}
	
	public Point(float _x, float _y)
	{
		x = _x;
		y = _y;
	}

	// copy constructor
	public Point(Point p)
	{
		x = p.x;
		y = p.y;
	}

	// returns circle centered at point of specified size
	public Shape toCircle(int size) 
	{
		return new Ellipse2D.Float(x-size/2, y-size/2, size, size);
	}
	
	// shorthand operator names that refer to fCompare
	public static boolean fEqual(float a, float b) 
	{ return Compare.compare(a, b) == 0; }
	public static boolean fLess(float a, float b) 
	{ return Compare.compare(a, b) == -1; }
	public static boolean fGreater(float a, float b) 
	{ return Compare.compare(a, b) == 1; }
	
	// Points are equal only if their components are
	@Override
	public boolean equals(Object obj)
	{
		return (obj instanceof Point &&
				fEqual(x, ((Point)obj).x) &&
				fEqual(y, ((Point)obj).y));
	}

	
	// SkillPoints are ordered based on x and y value
	@Override
	public int compareTo(Point s) {	
		if (fEqual(x, s.x) && fEqual(y, s.y)) return 0;
		if (fLess(x, s.x)) return -1;
		if (fEqual(x, s.x) && fLess(y, s.y)) return -1;
		return 1;
	}

	@Override
	public String toString()
	{
		return String.format("(%1.3f, %1.3f)", x, y); 
	}
}
