package pieChart;

import java.awt.Graphics2D;
import java.util.TreeMap;
import java.util.TreeSet;

import common.ColorMap;
import common.DefaultColorMap;

public class Pie extends TreeSet<PieItem> {
	String _filename = "";		
	double _duration = 1;
	double _diameter = 1;
	ColorMap _map = new DefaultColorMap();
	TreeMap<String, PieItem> _namedWedges = new TreeMap<String, PieItem>();
	
	Pie() {}

	Pie(String filename, double duration, double diameter, String... names) 
	{
		_filename = filename;
		_duration = duration;
		_diameter = diameter;
		
		double percent = 100d * 1d / names.length;
		for (String name : names) {
			add(name, percent);
		}
	}	
	
	Pie(String filename, double duration, double diameter, 
			ColorMap map, String... names) 
	{
		_filename = filename;
		_duration = duration;
		_diameter = diameter;
		_map = map;
		
		double percent = 100d * 1d / names.length;
		for (String name : names) {
			add(name, percent);
		}
	}

	// add(name, percent)
	//   name   : String = name of pie wedge
	//   percent: double = size of pie wedge, as percent of pie, from [0, 100]
	// adds specified pie wedge to pie, or increment wedge if already exists
	void add(String name, double percent)
	{
		PieItem wedge = null;

		// increment percent if wedge already exists
		if (_namedWedges.containsKey(name)) {
			wedge = _namedWedges.get(name);
			wedge._percent += percent;
		}

		// otherwise make new wedge
		else {
			wedge = new PieItem(name, percent, _diameter/2d, 
					_map.colorOf(name));
			_namedWedges.put(name, wedge);
			add(wedge);
		}
	}

	// toString()
	// returns string representation of pie, for debugging purposes
	@Override
	public String toString()
	{
		String rep = "Pie '" + _filename + "' {\n";
		for (PieItem wedge : this) {
			rep += "  " + wedge.toString() + "\n";
		}
		rep += "}";
		return rep;
	}

	// mergeColors()
	// merges neighbor pie wedges of the same color
	void mergeColors()
	{
		// move wedges to cache
		TreeSet<PieItem> cache = new TreeSet<PieItem>();
		for (PieItem wedge : this) cache.add(wedge);
		this.clear();

		// re-insert only merged wedges
		PieItem last = null;
		for (PieItem wedge : cache) {
			if (last == null) {
				last = wedge;
				continue;
			}

			// same color wedges get merged
			if (wedge._fill.equals(last._fill)) {
				PieItem piece = new PieItem(last);
				piece._percent += wedge._percent;
				last = piece;
				continue;
			}

			// otherwise store last wedge unmerged
			this.add(last);
			last = wedge;
		}
		this.add(last);
	}
	
	// paint(g2d)
	//   g2d: Graphics2D = graphics context
	// draws pie chart to screen
	// Note: assumes g2d has coordinate axis (0,0) at center of screen
	void paint(Graphics2D g2d)
	{
		// draw pie wedges in a circle starting east, going counter-clockwise
		double start = 0;
		for (PieItem wedge : this) {
			wedge.paint(g2d, start, true);
			double radians = (wedge._percent / 100d) * 2d * Math.PI;
			start += radians;
		}
	}
}
