package pieChart;
import java.awt.Color;
import java.util.ArrayList;


public class PieStats {

	ArrayList<CategoryInfo> _categories = new ArrayList<CategoryInfo>();
	
	public class WedgeInfo {
		String name = "";
		Color color = Color.black;
		double percent = 0;
		double points = 0;

		@Override
		public String toString() {
			String rep = "Wedge { name: " + name;
			rep += " points: " + String.format("%5.3f", points); 
			rep += " percent: " + String.format("%4.1f%%", percent) + "}";
			return rep;
		}
	}

	public static String toString(Color c)
	{
		String rep = "rgb(";
		rep += c.getRed() + ", ";
		rep += c.getGreen() + ", ";
		rep += c.getBlue();
		rep += ")";

		return rep;
	}
	
	public class CategoryInfo {
		Color color = Color.black;
		double percent = 0;
		double points = 0;
		ArrayList<WedgeInfo> wedges = new ArrayList<WedgeInfo>();

		@Override
		public String toString() {
			String rep = "Category {\n";
			rep += "color: " + PieStats.toString(color) + "\n";
			rep += "percent: " + String.format("%4.1f", percent) + "\n";
			rep += "points : " + String.format("%5.3f", points) + "\n";
			for (WedgeInfo winf : wedges) {
				rep += "  " + winf.toString() + "\n";
			}
			rep += "}";
			return rep;
		}
	}

	// collect statistics about sections and pieces of pie chart
	PieStats(Pie pie)
	{
		Color lastColor = null;
		CategoryInfo cat = new CategoryInfo();
		for (PieItem wedge : pie) {

			// switch to a new category if wedge changes colors
			if (lastColor == null || wedge._fill != lastColor) {
				lastColor = wedge._fill;
				cat = new CategoryInfo();
				_categories.add(cat);
			}

			// record category summary info
			cat.color = wedge._fill;
			cat.percent += wedge._percent;
			cat.points += (wedge._percent / 100d) * pie._duration;
			
			// copy info from wedges
			WedgeInfo winf = new WedgeInfo();
			winf.color = wedge._fill;
			winf.percent = wedge._percent;
			winf.points = (wedge._percent / 100d) * pie._duration;
			winf.name = wedge._name;
			cat.wedges.add(winf);
		}
	}

	@Override
	public String toString()
	{
		String rep = "PieInfo {\n";
		for (CategoryInfo cat : _categories) {
			rep += cat.toString() + "\n";
		}
		rep += "}";
		return rep;
	}
}
