package pieChart;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import javax.imageio.ImageIO;
import javax.swing.Timer;

import common.ColorMap;
import common.DefaultColorMap;


public class TestPie extends Applet implements ActionListener {

	double _diam = 150;           // pie chart diameter
	ArrayList<Pie> _pies = new ArrayList<Pie>();
	Iterator<Pie> _pie = null;
	Timer _timer = null;	
		
	@Override
	public void init() 
	{
		_timer = new Timer(256, this);
		_timer.start();
	}
	
	@Override
	public void start()
	{
		DefaultColorMap map = new DefaultColorMap();
		
		Pie pie = new Pie("2001.png", 1, _diam, map, "Perl", "XHTML", "JavaScript");
		_pies.add(pie);

		pie = new Pie("2002.png", 1, _diam, map, "Bash", "Unix", "Excel", "Gimp");
		_pies.add(pie);

		pie = new Pie("2003.png", 1d/3d, _diam, map, "Perl", "XHTML", "CSS", "Apache",
				"MySQL", "Access");
		_pies.add(pie);

		pie = new Pie("2003_5.png", 2d/3d, _diam, map, "C++", "VB", "InstallShield", 
				"Win32 Scripting", "Graph Viz");
		_pies.add(pie);

		pie = new Pie("2004.png", 0.5, _diam, map, "C", "Unix", "Visualization");
		_pies.add(pie);

		pie = new Pie("2005.png", 0.5, _diam, map, "Machining", "Soldering", "Wiring",
		"Electronic Repair");
		_pies.add(pie);

		pie = new Pie("2006.png", 1, _diam, map, "System Administration", 
				"Tech. Support", "Unix Scripting", "PC Hardware");
		_pies.add(pie);

		pie = new Pie("2006_9.png", 3, _diam, map, "Matlab", "C", "C++", 
				"Unix Scripting", "Visualization");
		_pies.add(pie);

		pie = new Pie("2009_12.png", 2, _diam, map, "C", "C++", "Haskell", 
				"Parallel Programming", "OpenGL");
		_pies.add(pie);
		
		pie = new Pie("2012-2013.png", 0.25, _diam, map, "C++", "CMake", "Win32", 
				"Qt" );
		_pies.add(pie);
		
		pie = new Pie("2013-2014.png", 11d/12d, _diam, map, "C", "Bash", 
				"Unix", "R", "Parallel Programming");
		_pies.add(pie);

		// condense several pie charts into one 
		double duration = 0, total_percent = 0;
		Pie tmp = new Pie("TEMP", duration, _diam, map);
		for (Pie p : _pies) {
			duration += p._duration;
			for (PieItem wedge : p) {
				tmp.add(wedge._name, wedge._percent * p._duration);				
				total_percent += wedge._percent * p._duration;
			}
		}
		
		// renormalize wedges to combined sum
		_diam = 240;
		pie = new Pie("summary.png", duration, _diam, map);
		for (PieItem wedge : tmp) {
			double new_percent = (wedge._percent / total_percent) * 100d;
			pie.add(wedge._name, new_percent);
		}
		_pies.add(pie);

		// print merged category info
		tmp = new Pie("SUMMARY", duration, _diam, map);
		for (PieItem wedge : pie) {
			tmp.add(map.nameOf(wedge._name), wedge._percent);
		}
		for (PieItem wedge : tmp) {
			System.out.print(wedge.toString());
			System.out.println(" years: " + duration * (wedge._percent/100));
		}
		//tmp.mergeColors();
		
		//System.out.println(new PieStats(pie).toString());
	}

	// initG2D(g)
	//   g: Graphics = graphics handle
	// set defaults for graphics and cast to Graphics2D instance
	private Graphics2D initG2D(Graphics g)
	{
		RenderingHints hints;
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                   RenderingHints.VALUE_ANTIALIAS_ON);
		hints.put(RenderingHints.KEY_RENDERING, 
                  RenderingHints.VALUE_RENDER_QUALITY);
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHints(hints);
		return g2d;
	}

	// flipY()
	// returns affine transform that flips Y axis to point positive downward
	public AffineTransform flipY()
	{
		float mat[] = {1, 0, 0, -1, 0, getHeight()-1};
		AffineTransform trans = new AffineTransform(mat);
		return trans;
	}	

	
	public static void drawString(Graphics2D g2d, String msg)
	{
		AffineTransform oldTrans = g2d.getTransform();

		float mat[] = {1, 0, 0, -1, 0, 0};
		g2d.transform(new AffineTransform(mat));
		g2d.drawString(msg, 0, 0);

		g2d.setTransform(oldTrans);
	}
	
	
	@Override
	// paint(g)
	//   g: Graphics = graphics context
	// paints series of pie charts either to screen or to named images
	public void paint(Graphics g)
	{
		// set up pies to advance when refresh is called
		if (_pie == null) _pie = _pies.iterator();
		if (!_pie.hasNext()) System.exit(0);
		Pie pie = _pie.next();
		boolean output_png = true; // set to 'true' to output .png file
		
		Graphics2D g2d = initG2D(g);

		// calculate size of pie bounding box
		double _margin = 256 - pie._diameter;   // empty margin around chart
		resize((int)(_diam+_margin), (int)(_diam+_margin));
		
		// create image canvas to render to
		BufferedImage img = new BufferedImage(getWidth(), getHeight(), 
				BufferedImage.TYPE_INT_ARGB);
		if (output_png) {
			g2d = img.createGraphics();		
			initG2D(g2d);
		}
		
		g2d.setPaint(new Color(255,255,255,0));
		g2d.fillRect(0, 0, getWidth(), getHeight());
		g2d.setClip(0, 0, getWidth(), getHeight());
		
		// flip coordinate space so we can draw in positive x,y plane
		AffineTransform oldTrans = g2d.getTransform();
		g2d.transform(flipY());
		g2d.setColor(Color.black);
		
		// translate origin to center of screen
		g2d.translate(getWidth()/2, getHeight()/2);

		g2d.setFont(new Font("Helevetica", Font.PLAIN, 20));
		pie.paint(g2d);
		
		// write image data to (existing) data file
		if (pie._filename == "summary.png") {

			// label with duration numbers
			if (false) {
				for (PieItem wedge : pie) {
					String amt = String.format("%2.1f", 
							pie._duration * (wedge._percent/100));
					wedge.label(g2d, amt, 5);
				}
			}
		}
		
		File out = new File("/Users/negaunt/Desktop/" + pie._filename);
		if (output_png) {
			try {
				ImageIO.write(img, "png", out);
				System.out.println("wrote: /Users/negaunt/Desktop/" 
						+ pie._filename);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		g2d.setTransform(oldTrans);
	}

	// actionPerformed(tick)
	//   tick: ActionEvent = a clock tick
	// perform animation work each time clock ticks
	@Override
	public void actionPerformed(ActionEvent tick) {
		repaint();
	}
}
