package sawtoothPlot;

import geometry.Point;
import java.applet.Applet;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.TreeMap;

import javax.imageio.ImageIO;
import javax.swing.Timer;

import common.PastelColorMap;



public class TestSawtooth extends Applet implements ActionListener {
	private static final long serialVersionUID = -3328378779003630667L;

	protected JobDB _work = null;
	protected JobDB _school = null;
	private TreeMap<String, List<String>> _categories = null;
	protected Timer _timer = null;
	boolean _halt = false;
	final int _margin_size = 30;      // margin width
	
	@Override
	public void init() 
	{
		final int msecs = 500;
		_work = new JobDB();
		_school = new JobDB();
		_categories = new TreeMap<String, List<String>>();
		_timer = new Timer(msecs, this);

		int width = 880;
		int height = width / 4;
		resize(width, height);
	}

	@Override
	public void start()
	{
		if (_work == null) return;
		_timer.start();

		// define category meta-data
		_categories.put("Languages", 
				Arrays.asList("Python", "Perl", "Java", "JavaScript","C++","C", 
						"Matlab", "Haskell", "VB", "CMake"));
		_categories.put("IT Systems",
				Arrays.asList("Bash", "Unix", "Linux", "Apache", "MySQL", "Sysadmin",
						"Tech Support", "PC Hardware", "HPC"));
		_categories.put("Graphics",
				Arrays.asList("Gimp", "Graphviz", "NCurses", "OpenGL",
						"Matlab", "Java2D", "Qt", "R"));
		_categories.put("Web Design",
				Arrays.asList("XHTML", "JavaScript", "CSS", "SVG", "JSP", 
						"Servlets"));
		_categories.put("Windows",
				Arrays.asList("Access", "Excel", "InstallShield", "Win32", 
						"VB"));
		_categories.put("Misc", 
				Arrays.asList("Electronics Assembly", "Machining", "Soldering"
						));

		// assign new color maps, if any
		_work._map = new PastelColorMap(_categories);
		
		// add skills and dates to job database
		_work.add("Web Design Intern", 2000, "Perl","XHTML","JavaScript");		
		_work.add("Unix Cluster Intern", 2001, "Bash","Unix","Excel","Gimp");
		_work.add("DB Coder", 2002.2f, 2002.7f, "Apache","MySQL","Perl","XHTML","CSS","Access");
		_work.add("Win32 Coder", 2003.2f, 2003.7f, "Graphviz","InstallShield","Win32",
				"VB","C++");
		_work.add("Teaching Assistant", 2004, 2004.5f, "C","Unix","NCurses");
		_work.add("Fab_R+D Tech", 2004.5f, 2005, "Machining","Soldering","Electronics Assembly");
		_work.add("Student Sysadmin", 2005, "Sysadmin","Tech Support",
				"Unix","PC Hardware");
		_work.add("Geophysics Intern", 2006, 2009, "Matlab","C","C++","Unix","HPC");
		_work.add("Graduate Research Assistant", 2009, 2012, "OpenGL", "C", "C++", "HPC", "Haskell");
		_work.add("App Coder", 2012.9f, 2013.25f, "C++", "CMake", "Win32", "Qt");
		_work.add("HPC Filesystem Developer", 2013.25f, 2014.09f, "C", "Bash", "Unix", "R");

		// calculate screen layout for data series
		_work.layout(getWidth()-2*_margin_size, getHeight()/2-_margin_size);

		// add skills and dates to school database
		_school._map = new PastelColorMap(_categories);
		_school.add("", 2000, 2006, "BSCS");
		_school.add("", 2006, 2010, "MSCS");
		_school.add("", 2010, 2012, "");
		
		// calculate screen layout for data series
		_school.layout(getWidth()-2*_margin_size, getHeight()/2-_margin_size);
		
		repaint();
	}	
	
	// initG2D(g)
	//   g: Graphics = graphics handle
	// set defaults for graphics and cast to Graphics2D instance
	private Graphics2D initG2D(Graphics g)
	{
		RenderingHints hints;
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                   RenderingHints.VALUE_ANTIALIAS_ON);
		hints.put(RenderingHints.KEY_RENDERING, 
                  RenderingHints.VALUE_RENDER_QUALITY);
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHints(hints);
		
		return g2d;
	}

	// flipY()
	// returns affine transform that flips and biases Y axis so that
	// (0, 0) is at the bottom left corner and 
	// (xmax, ymax) is at the top right corner of the drawing canvas
	public AffineTransform flipY()
	{
		// new_x = old_x;   new_y = (-1 * old_y) + (getHeight()-1);
		float mat[] = {1, 0, 0, -1, 0, getHeight()-1};
		AffineTransform trans = new AffineTransform(mat);
		return trans;
	}

	

	// drawLabel(g2d, font, x, y, label)
	//   g2d  : Graphics2D = graphics context
	//   font : Font       = text font
	//   x    : float      = x coordinate of baseline start
	//   y    : float      = y coordinate of baseline start
	//   label: String     = label string
	// draws plot label to top left corner of screen
	static void drawLabel(Graphics2D g2d, Font font, float x, float y, String label)
	{
		int font_height = (int)(0.6 * font.getSize());  // font height in pixels
		int line_spacing = (int)(0.8 * (float)font_height);  // space btwn lines
		Paint paint = g2d.getPaint();

		// draw text to screen, respecting line breaks
		g2d.setPaint(Color.black);
		g2d.setFont(font);
		String lines[] = label.split("\n");
		int baseline = (int)y - lines.length*(line_spacing + font_height);
		for (String line : lines) {
			g2d.drawString(line, x, baseline);
			baseline += line_spacing + font_height;
		}
		g2d.setPaint(paint);
	}

	public void drawCircle(Graphics2D g2d, Point p, int size)
	{
		Shape shape = p.toCircle(size);
		g2d.setColor(Color.white);
		g2d.fill(shape);
		g2d.setColor(Color.black);
		g2d.draw(shape);		

		Shape point = p.toCircle(size/2);
		g2d.fill(point);
	}
	
	@Override
	public void paint(Graphics g)
	{
		if (_work == null) return;
		_halt = true;  // only paint once
		
		Graphics2D g2d = initG2D(g);
		Color clearColor = Color.white;
		Color transColor = new Color(255,255,255,0);
		Font normalFont = new Font("Serif", Font.PLAIN, 15);
		Font boldFont = new Font("Serif", Font.BOLD, 15);
		final double years = Math.floor(_work._end - _work._begin);
		boolean output_png = true;  // change to 'true' to write .png file
		
		// create image canvas to render to (optional)
		BufferedImage img = new BufferedImage(getWidth(), getHeight(), 
				BufferedImage.TYPE_INT_ARGB);
		if (output_png) {
			g2d = img.createGraphics();		
			g2d = initG2D(g2d);
		}

		// clear screen with transparent white background
		g2d.setPaint(transColor);
		g2d.fillRect(0, 0, getWidth(), getHeight());		
		
		// flip Y axis so screen coordinates are in the first quadrant
		AffineTransform oldTrans = g2d.getTransform();
		g2d.transform(flipY());
		AffineTransform firstQuad = g2d.getTransform();

		// mask off lower half screen and margins, and paint job sawteeth
		g2d.setClip(_margin_size           , getHeight()/2-_margin_size, 
				    getWidth()-_margin_size, getHeight()  -_margin_size);
		g2d.translate(_margin_size, getHeight()/2);
		_work.paint(g2d);

		// paint job labels
		g2d.setTransform(oldTrans);
		g2d.translate(_margin_size, getHeight()/2-1);
		for (Job job : _work) {
			float xoff = ((job._end - job._begin) / (float)years) * getWidth();
			xoff /= 4;
			float yoff = ((job._end - job._begin) / (float)years) * 
					((getHeight())/2)+12;
			
			// tweaks
			if (job._title.contains("Win32 Coder")) xoff = 0;
			if (job._title.contains("Teaching Assistant")) xoff = 0;
			if (job._title.contains("Student Sysadmin")) xoff /= 3;
			if (job._title.contains("Fab_R+D Tech")) continue;
			if (job._title.contains("App Coder")) xoff = -10;
				
			Point label = Point.plus(job.first(), new Point(xoff, -yoff));
			String text = job._title.replace(' ', '\n');
			text = text.replace('_', ' ');
			Font jobFont = new Font("Serif", Font.PLAIN, 14);
			drawLabel(g2d, jobFont, label.x, label.y, text);
		}
		
		// mask off upper half of screen and margins, and paint school sawteeth
		g2d.setTransform(firstQuad);
		g2d.setClip(_margin_size           , _margin_size, 
				    getWidth()-_margin_size, getHeight()/2-_margin_size);
		g2d.setTransform(oldTrans);
		g2d.translate(_margin_size, getHeight()/2);
		_school.paint(g2d);
		
		// draw year (x-axis) tick marks
		g2d.setPaint(Color.black);
		g2d.setTransform(firstQuad);
		g2d.setClip(0, 0, getWidth(), getHeight());
		g2d.fillRect(_margin_size, getHeight()/2-2, getWidth()-2*_margin_size, 3);
		Point x_begin = new Point();
		Point x_end = new Point();
		for (int i = 0; i <= years; i++) {
			final float length = getWidth()-2*_margin_size;
			final float radius = 6,  diam = 2*radius; 
			float x = ((float)i/(float)years) * length + _margin_size - radius;
			float y = getHeight()/2 - radius;
			
			g2d.setColor(Color.white);
			g2d.fillOval((int)x, (int)y, (int)diam, (int)diam);
			g2d.setColor(Color.black);
			g2d.drawOval((int)x, (int)y, (int)diam, (int)diam);
			
			if (i == 0)     x_begin = new Point(x, y);
			if (i == years) x_end = new Point(x, y);
		}

		// draw first and last year (x-axis) text labels
		g2d.setTransform(oldTrans);
		Point x_begin_label = Point.plus(x_begin, new Point(-10, 45));
		Point x_end_label = Point.plus(x_end, new Point(-10, 45));
		drawLabel(g2d, normalFont, x_begin_label.x, x_begin_label.y, "2000");
		drawLabel(g2d, normalFont, x_end_label.x  , x_end_label.y  , "2012");
		
		// draw points when degrees granted
		g2d.setTransform(oldTrans);
		g2d.translate(_margin_size, getHeight()/2-1);
		Point bscs = _school.get("BSCS").last();
		Point bscs_label = Point.plus(bscs, new Point(-20, 40));
		Point mscs = _school.get("MSCS").last();
		Point mscs_label = Point.plus(mscs, new Point(-20, 40));
		drawCircle(g2d, bscs, 12);
		drawCircle(g2d, mscs, 12);

		// label points when degrees granted
		drawLabel(g2d, boldFont, bscs_label.x, bscs_label.y, "BS CS");
		drawLabel(g2d, boldFont, mscs_label.x, mscs_label.y, "MS CS");
		
		// write image data to (existing) data file (optional)
		if (output_png) {
			File out = new File("/Users/negaunt/Desktop/sawtooth.png");
			try {
				ImageIO.write(img, "png", out);
				System.out.println("wrote: /Users/negaunt/Desktop/sawtooth.png");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		g2d.setTransform(oldTrans);
	}

	// actionPerformed(tick)
	//   tick: ActionEvent = a clock tick
	// perform animation work each time clock ticks
	@Override
	public void actionPerformed(ActionEvent tick) {
		if (!_halt) repaint();
	}
}
