package stackedPlot;

import geometry.Point;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.util.Iterator;
import java.util.List;
import javax.swing.Timer;



public class TestStacked extends Applet implements ActionListener {
	private static final long serialVersionUID = -3328378779003630667L;

	protected SkillDB _db = null;
	protected Timer _timer = null;

	protected List<String> _skillNames = null;
	protected Iterator<String> _skill = null;
	protected String _last = null;
	
	@Override
	public void init() 
	{
		final int msecs = 500;
		_db = new SkillDB();
		_timer = new Timer(msecs, this);
		resize(1000, 100);
	}

	@Override
	public void start()
	{
		if (_db == null) return;
		_timer.start();
		
		System.out.println("populating db:");

		// add skills and dates to database
		_db.add(2001, "Perl","XHTML","JavaScript");
		_db.add(2002, "Bash","Unix","Excel","Gimp");
		_db.add(2003, "Apache","MySQL","Perl","XHTML","CSS","Access");
		_db.add(2003.5f, "Graph Viz","InstallShield","Win32 Scripting",
				"VB","C++");
		_db.add(2004, "C","Unix","Visualization");
		_db.add(2005, "Machining","Soldering","Wiring","Electronic Repair");
		_db.add(2006, "System Administration","Tech Support",
				"Unix Scripting","PC Hardware");
		_db.add(2006, 2009, "Matlab","C","C++","Unix Scripting",
		"Visualization");

		// add colors to database
		float red = 1.0f, green = 120f/360f, blue = 180f/360f;
		float orange = 41f/360f, yellow = 60f/360f, purple = 325f/360f;
		/**
		_db.setHue(red, "Perl", "XHTML", "JavaScript", "Gimp", "CSS");
		_db.setHue(orange, "Bash", "Unix", "Apache", "MySQL", "System Administration", 
				"Tech Support", "PC Hardware", "Unix Scripting");
		_db.setHue(yellow, "Excel", "Access", "InstallShield", "Win32 Scripting",
		"VB");
		_db.setHue(green, "Graph Viz", "Visualization");
		_db.setHue(blue, "C++", "C", "Matlab");
		_db.setHue(purple, "Machining", "Soldering", "Wiring", "Electronic Repair");
		 **/

		// add greyscale to database
		_db.setGray(0.4f, "Perl", "XHTML", "JavaScript", "Gimp", "CSS");
		_db.setGray(0.5f, "Bash", "Unix", "Apache", "MySQL", "System Administration", 
				"Tech Support", "PC Hardware", "Unix Scripting");
		_db.setGray(0.9f, "Excel", "Access", "InstallShield", "Win32 Scripting",
		"VB");
		_db.setGray(0.8f, "Graph Viz", "Visualization");
		_db.setGray(0.6f, "C++", "C", "Matlab");
		_db.setGray(0.7f, "Machining", "Soldering", "Wiring", "Electronic Repair");
		
		_skillNames = _db.skillNames();
		
		_db.normalize();		
		_db.offset();
		_db.normalize();
		_db.scale(getWidth(), getHeight()-1);
		
		repaint();
	}	
	
	// initG2D(g)
	//   g: Graphics = graphics handle
	// set defaults for graphics and cast to Graphics2D instance
	private Graphics2D initG2D(Graphics g)
	{
		RenderingHints hints;
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                   RenderingHints.VALUE_ANTIALIAS_ON);
		hints.put(RenderingHints.KEY_RENDERING, 
                  RenderingHints.VALUE_RENDER_QUALITY);
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHints(hints);
		
		return g2d;
	}

	// flipY()
	// returns affine transform that flips Y axis to point positive downward
	public AffineTransform flipY()
	{
		float mat[] = {1, 0, 0, -1, 0, getHeight()-1};
		AffineTransform trans = new AffineTransform(mat);
		return trans;
	}
	
	
	// drawLine(g2d, from, to)
	//   g2d  : Graphics2D = graphics context
	//   Point: from = starting point
	//   Point: to   = ending point
	// draws a line between from and to
	static void drawLine(Graphics2D g2d, Point from, Point to)
	{
		((Graphics)g2d).drawLine((int)from.x, (int)from.y, 
				(int)to.x, (int)to.y);
	}

	// drawPoint(g2d, p)
	//   g2d: Graphics2D = graphics context
	//   p  : Point      = an x,y point
	// draws point as a small dot
	static void drawPoint(Graphics2D g2d, Point p)
	{
		((Graphics)g2d).drawOval((int)p.x, (int)p.y, 1, 1);
	}

	// drawLabel(g2d, label)
	//   g2d  : Graphics2D = graphics context
	//   label: String     = plot label
	// draws plot label to top left corner of screen
	static void drawLabel(Graphics2D g2d, String label)
	{
		Paint paint = g2d.getPaint();
		float font_points = 14.5f;                      // font size in 'points'
		float font_height = 0.72f * font_points;        // size in pixels
		float font_width = 0.75f * font_points;         // average letter width
		int label_width = (int)(font_width * label.length());  // label width
		
		// draw white box around text
		g2d.setPaint(Color.white);
		g2d.fillRect(0, 0, label_width, (int)font_height);

		// draw text to screen
		g2d.setPaint(Color.black);
		g2d.setFont(new Font("Serif", Font.PLAIN, (int)font_points));
		g2d.drawString(label, 0, font_height);
		g2d.setPaint(paint);
	}
	
	@Override
	public void paint(Graphics g)
	{
		if (_db == null) return;
		
		// abstract rendering options are defined on a 100x100 square, so
		// we scale future operations to fill the (unknown) canvas size
		Graphics2D g2d = initG2D(g);

		// fill white background
		g2d.setPaint(Color.white);
		g2d.fillRect(0, 0, getWidth(), getHeight());
		
		// reset skill display sequence  
		if (_skill == null) _skill = _skillNames.iterator();
		
		// display offset skill
		String skill = _last;
		if (_skill.hasNext()) skill = _skill.next();
		Iterator<String> lasts = _skillNames.iterator();

		while (lasts.hasNext()) {
			String last = lasts.next();
			
			// flip and bias y axis for drawing
			AffineTransform oldTrans = g2d.getTransform();
			g2d.transform(flipY());

			// draw offset skill in HSB fill color
			g2d.setPaint(_db.getColor(last));
			_db.paint(g2d, last);

			// restore y axis and draw skill label
			g2d.setTransform(oldTrans);
			drawLabel(g2d, skill);

			if (last == skill) break;
		}
		_last = skill;
	}

	// actionPerformed(tick)
	//   tick: ActionEvent = a clock tick
	// perform animation work each time clock ticks
	@Override
	public void actionPerformed(ActionEvent tick) {
		repaint();
	}
}
